(function(ns) {
    var _earth = null,
        _dataProcess = null,
        _seTrackMgr = null,
        _roles = null,
        _data = {
            isRoaming: false,
            readyRoaming: false,
            chead: 180,
            ctilt: 15,
            cmodeType: 2,
            cobsrHead: 180
        };


    //main
    function _initRoam(earth, dataProcess) {
        _earth = earth;
        _dataProcess = dataProcess

        _seTrackMgr = new ns.STAMP.SeTrackManager({
            earth: _earth,
            dataProcess: _dataProcess
        });

        _initUI();
        _addEventListeners();
    }

    function _destroyRoam() {
        _removeEventListeners();
    }

    function _initUI() {
        _initRoles();
    }

    function _addEventListeners() {
        function _onKeyDown(e) {
            if (!_checkKeyCode(e.keyCode)) {
                e.preventDefault();
            }

            if (e.keyCode == 13) {
                if (!($('#roamPanel>#roamContainer').find("#obhead").attr("disabled")) &&
                    !($('#roamPanel>#roamContainer').find("#obtilt").attr("disabled"))) {
                    _setHeadTilt();
                }
            }
        }

        function _setHeadTilt() {
            var roleId = $('#roamPanel>#roamContainer').find('#roles').val();
            if (roleId == '') {
                return;
            }

            _data.chead = $('#roamPanel>#roamContainer').find('#obhead').val();
            _data.ctilt = $('#roamPanel>#roamContainer').find('#obtilt').val();

            _data.chead = _seTrackMgr.calcHeading({
                heading: _data.chead
            });
            _data.ctilt = _seTrackMgr.calcTilt({
                tilt: _data.ctilt
            });

            $('#roamPanel>#roamContainer').find('#obhead').val(_data.chead);
            $('#roamPanel>#roamContainer').find('#obtilt').val(_data.ctilt);

            _earth.GlobeObserver.InitThirdTrack(_data.chead, _data.ctilt);

            _seTrackMgr.startTrack({
                dynamicId: roleId,
                type: _data.cmodeType
            });
        }

        $('#roamPanel>#roamContainer').find('#enter').bind('click', function(e) {
            try {
                var roleId = $('#roamPanel>#roamContainer').find('#roles').val();
                if (roleId == '') {
                    return;
                }

                _data.isRoaming = false;
                _data.readyRoaming = true;
                _updateState($('#roamPanel>#roamContainer'), _data);

                _seTrackMgr.enter({
                    dynamicId: roleId,
                    heading: _data.chead,
                    tilt: _data.ctilt,
                    type: _data.cmodeType,
                    onDynamicLoaded: function() {
                        _data.isRoaming = true;
                        _data.readyRoaming = false;
                        _updateState($('#roamPanel>#roamContainer'), _data);

                        $('#roamPanel>#roamContainer').find('#out')[0].blur();
                        $('#roamPanel>#roamContainer').find('#out').removeAttr('type');
                    }
                });
            } catch (e) {

            }
        });

        $('#roamPanel>#roamContainer').find('#out').bind('click', function(e) {
            try {
                var roleId = $('#roamPanel>#roamContainer').find('#roles').val();
                if (roleId == '') {
                    return;
                }

                _seTrackMgr.out({
                    dynamicId: roleId
                });

                _data.isRoaming = false;
                _data.readyRoaming = false;
                _updateState($('#roamPanel>#roamContainer'), _data);
            } catch (e) {

            }
        });

        $('#roamPanel>#roamContainer').find('#modeType :radio').bind('click', function(e) {
            var rdo = e.currentTarget;

            try {
                $('#roamPanel>#roamContainer').find('#modeType :radio').removeAttr('checked');
                $('#roamPanel>#roamContainer').find('#modeType :radio').each(function(i, n) {
                    n.checked = false;
                });
                $(rdo).attr('checked', 'checked');
                rdo.checked = true;

                _data.cmodeType = $(rdo).val();

                if (_data.cmodeType == ns.STAMP.seEnums.seTrackingModeType.thirdPerson) {
                    _earth.GlobeObserver.InitThirdTrack(_data.chead, _data.ctilt);
                }

                var roleId = $('#roamPanel>#roamContainer').find('#roles').val();
                if (roleId == '') {
                    return;
                }

                _seTrackMgr.startTrack({
                    dynamicId: roleId,
                    type: _data.cmodeType
                });

                _updateState($('#roamPanel>#roamContainer'), _data);
            } catch (e) {

            }
        });

        $('#roamPanel>#roamContainer').find('#obsrHead :radio').bind('click', function(e) {
            var rdo = e.currentTarget;

            try {
                $('#roamPanel>#roamContainer').find('#obsrHead :radio').removeAttr('checked');
                $('#roamPanel>#roamContainer').find('#obsrHead :radio').each(function(i, n) {
                    n.checked = false;
                });
                $(rdo).attr('checked', 'checked');
                rdo.checked = true;

                _data.cobsrHead = $(rdo).val();
                var roleId = $('#roamPanel>#roamContainer').find('#roles').val();
                if (roleId == '') {
                    return;
                }

                if (_data.cobsrHead != 'NaN') {
                    _earth.GlobeObserver.ChangeThirdTrackHeading(_data.cobsrHead, roleId);
                } else {
                    _setHeadTilt();
                }

                _updateState($('#roamPanel>#roamContainer'), _data);
            } catch (e) {

            }
        });

        $('#roamPanel>#roamContainer').find('#obhead').bind('keydown', _onKeyDown);

        $('#roamPanel>#roamContainer').find('#obtilt').bind('keydown', _onKeyDown);

        $('#roamPanel>#roamContainer').find('#obhead').bind('change', _setHeadTilt);

        $('#roamPanel>#roamContainer').find('#obtilt').bind('change', _setHeadTilt);
    }

    function _removeEventListeners() {
        $('#roamPanel>#roamContainer').find('#enter').unbind('click');
        $('#roamPanel>#roamContainer').find('#out').unbind('click');

        $('#roamPanel>#roamContainer').find('#modeType :radio').unbind('click');
        $('#roamPanel>#roamContainer').find('#obsrHead :radio').unbind('click');

        $('#roamPanel>#roamContainer').find('#obhead').unbind('keydown');
        $('#roamPanel>#roamContainer').find('#obtilt').unbind('keydown');

        $('#roamPanel>#roamContainer').find('#obhead').unbind('change');
        $('#roamPanel>#roamContainer').find('#obtilt').unbind('change');
    }

    function _initRoles() {
        _roles = [];
        _seTrackMgr.getDynamicObject({
            onDynamicObject: function(landRole) {
                if (landRole) {
                    _roles.push({
                        id: landRole.Guid,
                        name: landRole.Name
                    });
                }
            },
            onDynamicListLoaded: function(roles) {
                var ops = '';
                for (var i = 0, l = _roles.length; i < l; i++) {
                    ops += '<option value="' + _roles[i].id + '">' + _roles[i].name + '</option>';
                }
                $('#roamPanel>#roamContainer').find('#roles').html(ops);
                $('#roamPanel>#roamContainer').find('#roles').selectpicker();

                _updateState($('#roamPanel>#roamContainer'), _data);
            }
        });
    }

    function _updateState(el, data) {
        $(el).find('#modeType :radio').each(function(i, n) {
            if ($(n).val() == data.cmodeType) {
                $(n).attr('checked', 'checked');
                $(n)[0].checked = true;
            } else {
                $(n).removeAttr('checked');
                $(n)[0].checked = false;
            }
        });

        $(el).find('#obsrHead :radio').each(function(i, n) {
            if ($(n).val() == data.cobsrHead) {
                $(n).attr('checked', 'checked');
                $(n)[0].checked = true;
            } else {
                $(n).removeAttr('checked');
                $(n)[0].checked = false;
            }
        });

        $(el).find('#obhead').val(_data.chead);
        $(el).find('#obtilt').val(_data.ctilt);

        if (!data.isRoaming) {
            if (!data.readyRoaming) {
                $(el).find('#roles').removeAttr('disabled');
                $(el).find('#roles').prop('disabled', false);
                $(el).find('#roles').selectpicker('refresh');
                $(el).find('#enter').removeAttr('disabled');
                $(el).find('#out').attr('disabled', 'disabled');
            } else {
                $(el).find('#roles').attr('disabled', 'disabled');
                $(el).find('#roles').prop('disabled', true);
                $(el).find('#roles').selectpicker('refresh');
                $(el).find('#enter').attr('disabled', 'disabled');
                $(el).find('#out').attr('disabled', 'disabled');
            }

            $(el).find('#modeType').attr('disabled', 'disabled');

            $(el).find('#obsrHead').attr('disabled', 'disabled');

            $(el).find('#headTilt').attr('disabled', 'disabled');
            $(el).find('#headTilt input').attr('readonly', 'readonly');
        } else {
            $(el).find('#roles').attr('disabled', 'disabled');
            $(el).find('#roles').prop('disabled', true);
            $(el).find('#roles').selectpicker('refresh');
            $(el).find('#enter').attr('disabled', 'disabled');
            $(el).find('#out').removeAttr('disabled');

            $(el).find('#modeType').removeAttr('disabled');

            if ($(el).find('#thirdView').attr('checked') == 'checked') {
                $(el).find('#obsrHead').removeAttr('disabled');

                if ($(el).find('#custom').attr('checked') == 'checked') {
                    $(el).find('#headTilt').removeAttr('disabled');
                    $(el).find('#headTilt input').removeAttr('readonly');
                } else {
                    $(el).find('#headTilt').attr('disabled', 'disabled');
                    $(el).find('#headTilt input').attr('readonly', 'readonly');
                }
            } else {
                $(el).find('#obsrHead').attr('disabled', 'disabled');

                $(el).find('#headTilt').attr('disabled', 'disabled');
                $(el).find('#headTilt input').attr('readonly', 'readonly');
            }
        }

        $(el).find('#out')[0].blur();
        $(el).find('#out').removeAttr('type');
    }

    function _checkKeyCode(keyCode) {
        if (isNaN(keyCode)) {
            return false;
        }

        if (keyCode >= 48 && keyCode <= 57) {
            //主键盘数字键
            return true;
        }
        if (keyCode >= 96 && keyCode <= 105) {
            //小键盘数字键
            return true;
        }
        if (keyCode == 13) {
            //主键盘回车键
            return true;
        }
        if (keyCode == 108) {
            //小键盘回车键
            return true;
        }
        if (keyCode == 189) {
            //主键盘'-'号
            return true;
        }
        if (keyCode == 109) {
            //小键盘'-'号
            return true;
        }
        if (keyCode == 8) {
            //backspace
            return true;
        }
        if (keyCode == 46) {
            //delete
            return true;
        }
        if (keyCode >= 37 && keyCode <= 40) {
            //方向键
            return true;
        }
        if (keyCode == 190) {
            //主键盘'.'
            return true;
        }
        if (keyCode == 110) {
            //小键盘'.'
            return true;
        }

        return false;
    }


    if (!ns.STAMP.initer) {
        ns.STAMP.initer = {};
    }
    ns.STAMP.initer.initRoam = _initRoam;
    ns.STAMP.initer.destroyRoam = _destroyRoam;
})(window);
